// capture.ts
// main entry point for Socket Mobile CaptureJS SDK
//
// Copyright © 2019 Socket Mobile, Inc.
// ignore_for_file: unnecessary_this, unnecessary_null_comparison

import 'dart:async';

import '../capture_flutter_beta.dart';
import './capture_options.dart';

String defaultHost = "http://127.0.0.1:18481";

class Capture {
  Transport? transport;
  String host = defaultHost;
  int? clientOrDeviceHandle;
  int? transportHandle;
  Function? onEventNotification;
  Capture? rootCapture;
  Logger? logger;
  bool? iOS;

  Capture(
      [this.logger,
      this.clientOrDeviceHandle,
      this.transportHandle,
      this.onEventNotification,
      this.transport]);

  Map<String, dynamic> toJson() => {
        'transport': transport,
        'host': host,
        'clientOrDeviceHandle': clientOrDeviceHandle,
        'transportHandle': transportHandle,
        'rootCapture': rootCapture
      };

  Future<int?> transportHelper(AppInfo appInfo, Function eventNotification,
      [CaptureOptions? options]) async {
    if (options != null) {
      this.transport =
          options.transport ?? Transport().getTransport(this.logger);
      this.host = options.host ?? defaultHost;
    } else {
      this.transport = Transport().getTransport(this.logger);
    }

    try {
      int? _tryOpen = await this.transport!.openClient(host, appInfo,
          (dynamic event, int handle) {
        return eventNotification(event, handle);
      });
      this.clientOrDeviceHandle = _tryOpen;
      return this.clientOrDeviceHandle;
    } on CaptureException {
      rethrow;
    } catch (e) {
      throw CaptureException(-32602, 'Something went wrong.', e.toString());
    }
  }

  Future<int?> openClient(AppInfo appInfo, Function eventNotification,
      [CaptureOptions? options]) {
    return transportHelper(appInfo, eventNotification, options);
  }

  Future<int> close([int? handle]) async {
    if (this.transport != null && this.clientOrDeviceHandle != null) {
      try {
        int? hndl = handle ?? this.clientOrDeviceHandle;
        await this.transport?.close(hndl);
        this.rootCapture = null;
        this.transport = null;
        this.clientOrDeviceHandle = null;
        return SktErrors.ESKT_NOERROR;
      } on CaptureException {
        rethrow;
      } catch (e) {
        throw CaptureException(-32602, 'Something went wrong.', e.toString());
      }
    } else {
      throw CaptureException(
          SktErrors.ESKT_NOTINITIALIZED, 'Transport is not initialized');
    }
  }

  Future openDevice(String guid, Capture? capture) async {
    if (capture == null) {
      throw CaptureException(SktErrors.ESKT_INVALIDPARAMETER,
          'The provided parameter is invalid', 'NO CAPTURE INSTANCE');
    } else {
      this.rootCapture = capture;
      this.transport = capture.transport;
      this.transportHandle = capture.transportHandle;
      try {
        dynamic value = await this
            .transport
            ?.openDevice(this.rootCapture!.clientOrDeviceHandle, guid);
        this.clientOrDeviceHandle = value;
        return SktErrors.ESKT_NOERROR;
      } on CaptureException {
        rethrow;
      } catch (e) {
        throw CaptureException(-32602, 'Something went wrong.', e.toString());
      }
    }
  }

  Future<CaptureProperty> getProperty(CaptureProperty property) async {
    if (this.transport != null) {
      return this.transport!.getProperty(this.clientOrDeviceHandle, property);
    }

    throw CaptureException(
        SktErrors.ESKT_NOTINITIALIZED, 'Transport is not initialized');
  }

  Future<CaptureProperty> setProperty(CaptureProperty property) async {
    if (this.transport != null) {
      return this.transport!.setProperty(this.clientOrDeviceHandle, property);
    }

    throw CaptureException(
        SktErrors.ESKT_NOTINITIALIZED, 'Transport is not initialized');
  }
}
